unit PwFile;

{$MODE Delphi}

interface

uses SysUtils, LCLIntf, LCLType, LMessages, Messages, Classes, Graphics, Controls, Forms, Dialogs, ComCtrls;

type
  TFileEvent = procedure of object;
  TPathEvent = procedure of object;
  TStartEvent = procedure(var Cancel: boolean) of object;
  TStopEvent = procedure of object;
  TFileRunner = class(TComponent)
  private
    { Private-Deklarationen }
    FDir: string;
    FMask: string;
    FFlags: Integer;
    FOnFile: TFileEvent;
    FOnStart: TStartEvent;
    FOnStop: TStopEvent;
    FRunning: Boolean;
    FSubDirs: Boolean;
    //FFileName:      String;
    FPath: string;
    FUserBreak: Boolean;
    FOnPath: TPathEvent;
    FOnPathReady: TPathEvent;
    FTV: TCustomTreeView;
    FFileList: TStrings;
    function GetReadOnly: Boolean;
    procedure SetReadOnly(Value: Boolean);
    function GetHidden: Boolean;
    procedure SetHidden(Value: Boolean);
    function GetSYSFile: Boolean;
    procedure SetSYSFile(Value: Boolean);
    function GetVolumeID: Boolean;
    procedure SetVolumeID(Value: Boolean);
    function GetDirectory: Boolean;
    procedure SetDirectory(Value: Boolean);
    function GetArchive: Boolean;
    procedure SetArchive(Value: Boolean);

    function GetFileName: string;
    function GetFileExt: string;
    function GetPath: string;
    function GetSize: Longint;
    function GetTime: TDateTime;
    function GetFileFlags: Longint;

  protected
    { Protected-Deklarationen }
    procedure DoRun(thePath: string);
  public
    { Public-Deklarationen }
    FSR: TSearchRec;
    constructor Create(AOwner: TComponent); override;
    procedure Run;
    procedure Stop;
    property Path: string read GetPath;
    property FileName: string read GetFileName;
    property FileExt: string read GetFileExt;
    property FileSize: Longint read GetSize;
    property FileTime: TDateTime read GetTime;
    property FileFlags: Longint read GetFileFlags;
    property Running: Boolean read FRunning;
    property SR: TSearchRec read FSR write FSR;
    property TV: TCustomTreeView read FTV write FTV;
    property UserBreak: Boolean read FUserBreak;
    property FileList: TStrings read FFileList write FFileList;
  published
    { Published-Deklarationen }
    property Dir: string read FDir write FDir;
    property Mask: string read FMask write FMask;
    property aReadOnly: Boolean read GetReadOnly write SetReadOnly;
    property aHidden: Boolean read GetHidden write SetHidden;
    property aSYSFile: Boolean read GetSYSFile write SetSYSFile;
    property aVolumeID: Boolean read GetVolumeID write SetVolumeID;
    property aArchive: Boolean read GetArchive write SetArchive;
    property aDirectory: Boolean read GetDirectory write SetDirectory;
    property SubDirs: Boolean read FSubDirs write FSubDirs;
    property OnPath: TPathEvent read FOnPath write FOnPath;
    property OnFile: TFileEvent read FOnFile write FOnFile;
    property OnPathReady: TPathEvent read FOnPathReady write FOnPathReady;
    property OnStart: TStartEvent read FOnStart write FOnStart;
    property OnStop: TStopEvent read FOnStop write FOnStop;
  end;

  procedure Register;

implementation

//uses Keofunc, Keomess;

type
  TCustomTVCousin = class(TCustomTreeView)
  public
    property Items;
  end;

{ TFileRunner }

{--------------------------------------TFileRunner----------------------Create-}

constructor TFileRunner.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRunning := False;
  FOnFile := nil;
  FUserBreak := False;
  FFileList := nil;
  FTV := nil;
end;

{--------------------------------------TFileRunner-------------------------Run-}

procedure TFileRunner.Run;
var
  savFlags: Byte;
  Cancel: Boolean;
begin
  if (not (Assigned(FOnFile) or Assigned(FOnPath) or
    Assigned(TV) or Assigned(FFileList))) or FRunning then Exit;
  if Assigned(TV) then
  begin
    TCustomTVCousin(TV).Items.BeginUpdate;
    TCustomTVCousin(TV).Items.Clear;
  end;
  if Assigned(FFileList) then
    FFileList.Clear;
  FRunning := True;
  Cancel := False;
  FUserBreak := False;
  savFlags := FFlags;
  try
    if Assigned(FOnStart) then FOnStart(Cancel);
    if Cancel then Exit;
    DoRun(Dir);
  finally
    FRunning := False;
    FFlags := savFlags;
    if Assigned(FOnStop) then FOnStop;
    if Assigned(TV) then
    begin
      TCustomTVCousin(TV).Items.EndUpdate;
    end;
  end;
end;

{--------------------------------------TFileRunner-----------------------DoRun-}

procedure TFileRunner.Stop;
begin
  FRunning := False;
  FUserBreak := True;
end;

{--------------------------------------TFileRunner-----------------------DoRun-}

procedure TFileRunner.DoRun(thePath: string);
const
  Cnt: Longint = 0;
var
  F: TSearchRec;
  res: Integer;
  Dirs: TStringList;
  n, n1, n2: TTreeNode;
begin
  {Dateien}
  thePath := SysUtils.IncludeTrailingBackslash(thePath);
  Dirs := TStringList.Create;
  try
    Dirs.AddObject(thePath, nil);
    while FRunning and (Dirs.Count > 0) do begin

      {Gehe alle Dateien durch}
      FPath := Dirs.Strings[0];
      n := TTreeNode(Dirs.Objects[0]);
      if Assigned(FOnPath) then FOnPath;
      res := FindFirst(Path + Mask, FFlags, F);
      try
        while FRunning and (res = 0) do begin
          if (F.Name[1] <> '.') then
          begin
            FSR := F;
            try
              if Assigned(FOnFile) then
                FOnFile;
              if Assigned(TV) then
              begin
                n2 := TCustomTVCousin(TV).Items.AddChild(n, FSR.Name);
                n2.ImageIndex := 0;
                n2.SelectedIndex := 1;
              end;
              if Assigned(FFileList) then
                FFileList.Add(Path + FSR.Name);
            except
              Stop;
            end;
          end;
          res := FindNext(F);
        end;
      finally
        SysUtils.FindClose(F);
      end;
      if Assigned(FOnPathReady) then FOnPathReady;

      {Fge alle Verzeichnisse zu Dirs}
      if SubDirs then
      begin
        n1 := n;
        res := FindFirst(Path + '*.*', faDirectory, F);
        try
          while FRunning and (res = 0) do
          begin
            if (F.Attr and faDirectory = faDirectory) and (F.Name[1] <> '.') then
            begin
              if Assigned(TV) then
              begin
                n := TCustomTVCousin(TV).Items.AddChild(n1, F.Name);
                n.ImageIndex := 2;
                n.SelectedIndex := 3;
              end;
              Dirs.AddObject(Path + F.Name + '\', n);
            end;
            res := FindNext(F);
          end;
        finally
          SysUtils.FindClose(F);
        end;
      end;
      Dirs.Delete(0);
      inc(Cnt);
    end;
  finally
    Dirs.Free;
  end;
end;

{--------------------------------------TFileRunner-----------------GetReadOnly-}

function TFileRunner.GetReadOnly: Boolean;
begin
  Result := (FFlags and faReadOnly) <> 0;
end;

{--------------------------------------TFileRunner-----------------SetReadOnly-}

procedure TFileRunner.SetReadOnly(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faReadOnly
  else FFlags := FFlags and (not faReadOnly);
end;

{--------------------------------------TFileRunner-------------------GetHidden-}

function TFileRunner.GetHidden: Boolean;
begin
  Result := (FFlags and faHidden) <> 0;
end;

{--------------------------------------TFileRunner-------------------SetHidden-}

procedure TFileRunner.SetHidden(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faHidden
  else FFlags := FFlags and (not faHidden);
end;

{--------------------------------------TFileRunner------------------GetSYSFile-}

function TFileRunner.GetSYSFile: Boolean;
begin
  Result := (FFlags and faSYSFile) <> 0;
end;

{--------------------------------------TFileRunner------------------SetSYSFile-}

procedure TFileRunner.SetSYSFile(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faSYSFile
  else FFlags := FFlags and (not faSYSFile);
end;

{--------------------------------------TFileRunner-----------------GetVolumeID-}

function TFileRunner.GetVolumeID: Boolean;
begin
  Result := (FFlags and faVolumeID) <> 0;
end;

{--------------------------------------TFileRunner-----------------SetVolumeID-}

procedure TFileRunner.SetVolumeID(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faVolumeID
  else FFlags := FFlags and (not faVolumeID);
end;

{--------------------------------------TFileRunner----------------GetDirectory-}

function TFileRunner.GetDirectory: Boolean;
begin
  Result := (FFlags and faDirectory) <> 0;
end;

{--------------------------------------TFileRunner----------------SetDirectory-}

procedure TFileRunner.SetDirectory(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faDirectory
  else FFlags := FFlags and (not faDirectory);
end;

{--------------------------------------TFileRunner------------------GetArchive-}

function TFileRunner.GetArchive: Boolean;
begin
  Result := (FFlags and faArchive) <> 0;
end;

{--------------------------------------TFileRunner------------------SetArchive-}

procedure TFileRunner.SetArchive(Value: Boolean);
begin
  if Value
    then FFlags := FFlags or faArchive
  else FFlags := FFlags and (not faArchive);
end;

function TFileRunner.GetFileName: string;
begin
  if not FRunning then begin Result := '';
    exit;
  end;
  Result := FSR.Name;
end;

function TFileRunner.GetFileExt: string;
begin
  if not FRunning then begin Result := '';
    exit;
  end;
  Result := ExtractFileExt(FSR.Name);
end;

function TFileRunner.GetPath: string;
begin
  if not FRunning then begin Result := '';
    exit;
  end;
  Result := FPath;
end;

function TFileRunner.GetSize: Longint;
begin
  if not FRunning then begin Result := 0;
    exit;
  end;
  Result := FSR.Size;
end;

function TFileRunner.GetTime: TDateTime;
begin
  if not FRunning then begin Result := 0;
    exit;
  end;
  try
    Result := FileDateToDateTime(FSR.Time);
  except
    Result := 0;
  end;
end;


procedure Register;
begin
  RegisterComponents('Patchwork', [TFileRunner]);
end;


function TFileRunner.GetFileFlags: Longint;
begin
  Result := FSR.Attr;
end;

end.

